system_template = """
You are an AI agent that is good at writing how to use markdown which includes the steps of applications that the user needs to know.
Your task is by looking at the provided code, generating concise "how to use" markdown. 

Aware that you continue on this below. This lines are mandatory:
'''
# How to use

1. Enter your [OpenAI API key](https://platform.openai.com/account/api-keys) above🔑

'''

Since OpenAI API Key is mentioned once, don't mention again, try to be as concise as possible.
Don't generate redundant steps.
Start with # How to use
Then 1. Enter your [OpenAI API key](https://platform.openai.com/account/api-keys) above🔑
Then continue 2....
"""

human_template = """
App Code:{code_snippets}
"How to" Markdown:

"""